const fs = require('fs')
const ora = require('ora')
const path = require('path')
const fse = require('fs-extra')
const rm = require('rimraf').sync
const downloadRepo = require('download-git-repo')

const logger = require('./logger')

const mkdir = fs.mkdirSync
const exists = fs.existsSync

const cleanPath = dir => {
    if (exists(dir)) {
        rm(dir)
    }
    mkdir(dir)
}

/**
 * 
 * @param {*} name component name
 * @param {*} dir save path
 * @param {*} cb 
 */
function download(name, dir, done) {
    if (!exists(dir)) mkdir(dir)
    const repoPath = `gitlab:git.mysoft.com.cn:paas-component/${name}`
    const spinner = ora('downloading template')
    spinner.start()

    // 准备临时目录存放下载文件
    const tempDir = path.join(__dirname, '../.download-temp', name)
    cleanPath(tempDir)

    // 先下载到临时目录，然后复制到目标目录 dir
    // 为了保证目录名和组件包名一致
    downloadRepo(repoPath, tempDir, { clone: true }, err => {
        spinner.stop()
        if (err) logger.fatal('Failed to download repo from ' + repoPath + ': ' + err.message.trim())

        const name = require(path.join(tempDir, 'package.json')).name
        const target = path.join(dir, name)
        if (exists(target)) {
            rm(target)
        }
        fse.copySync(tempDir, target)
        done()
    })
}

module.exports = download
